@@ -1,6 +1,6 @@
 !=======================================================================
 !
-!  PROGRAM  PHASE/0 2014.02 ($Rev: 389 $)
+!  PROGRAM  PHASE/0 2016.01 ($Rev: 633 $)
 !
 !  SUBROUINE: forcrsv, forc_cnst, copy_to_cpd_forc_and_forcp, 
 !             cnstrnt, rplcps, rbinuc, change_of_coordinate_system
@@ -31,12 +31,12 @@
 !  Consortium.
 !   The activity of development of this program set has been supervised by Takahisa Ohno.
 !
-! $Id: b_Ionic_System.f90 389 2014-07-01 10:06:54Z jkoga $
+! $Id: b_Ionic_System.f90 633 2020-12-01 05:11:03Z jkoga $
 subroutine forcrsv(natm,ityp,imdtyp,iwei,amion,cpd_l,tkb,nrsv,imdalg &
      & ,frsv,iw_cnst)
 !=======================================================================
 !
-!  PROGRAM  PHASE/0 2014.02 ($Rev: 389 $)
+!  PROGRAM  PHASE/0 2016.01 ($Rev: 633 $)
 !
 !  SUBROUINE: forcrsv
 !
@@ -119,7 +119,7 @@ subroutine forc_cnst(natm,ityp,amion,cps,cnst_typ,nfcatm,ia_cnst,imdtyp &
      &, sgmc,gca)
 !=======================================================================
 !
-!  PROGRAM  PHASE/0 2014.02 ($Rev: 389 $)
+!  PROGRAM  PHASE/0 2016.01 ($Rev: 633 $)
 !
 !  SUBROUINE: forc_cnst
 !
@@ -197,7 +197,7 @@ end subroutine forc_cnst
 subroutine copy_to_cpd_forc_and_forcp(katm,natm,cpd_l,forc_l,cpd,forc,forcp)
 !=======================================================================
 !
-!  PROGRAM  PHASE/0 2014.02 ($Rev: 389 $)
+!  PROGRAM  PHASE/0 2016.01 ($Rev: 633 $)
 !
 !  SUBROUINE: copy_to_cpd_forc_and_forcp
 !
@@ -231,7 +231,7 @@ subroutine cnstrnt(natm,ityp,amion,cps,cnst_typ,nfcatm,ia_cnst,imdtyp &
      &, sgmc,sigma)
 !=======================================================================
 !
-!  PROGRAM  PHASE/0 2014.02 ($Rev: 389 $)
+!  PROGRAM  PHASE/0 2016.01 ($Rev: 633 $)
 !
 !  SUBROUINE: cnstrnt
 !
@@ -312,7 +312,7 @@ end subroutine cnstrnt
 subroutine rplcps(cps,ityp,mode,natm2,natm,iwei)
 !=======================================================================
 !
-!  PROGRAM  PHASE/0 2014.02 ($Rev: 389 $)
+!  PROGRAM  PHASE/0 2016.01 ($Rev: 633 $)
 !
 !  SUBROUINE: rplcps
 !
@@ -349,7 +349,7 @@ end subroutine rplcps
 subroutine rbinuc(pos,natm)
 !=======================================================================
 !
-!  PROGRAM  PHASE/0 2014.02 ($Rev: 389 $)
+!  PROGRAM  PHASE/0 2016.01 ($Rev: 633 $)
 !
 !  SUBROUINE: rbinuc
 !
@@ -1257,3 +1257,521 @@ end subroutine fd_supercell_symmetrize
 !!$  end function operatorExists
 !!$!-----------------------------------------------------------------------------------
 !!$end subroutine fd_symmetrize_sc2
+
+subroutine set_iatomn_default(elen,ntyp, speciesname, iatomn)
+  use m_Const_Parameters, only : DP
+  implicit none
+  integer, intent(in) :: elen,ntyp
+  character(len=elen), dimension(ntyp), intent(in) :: speciesname
+  real(kind=DP), dimension(ntyp), intent(out) :: iatomn
+  character(len=elen), allocatable :: ename_table(:)
+  integer :: it,ie,pos,ii
+  integer :: nmax_elem = 120
+
+  allocate( ename_table( nmax_elem ) );ename_table=''
+!  call set_table( 4,  1, 'H', 'He', 'Li', 'Be', 'B' )
+  ename_table(1)  = 'H'
+  ename_table(2)  = 'He'
+  ename_table(3)  = 'Li'
+  ename_table(4)  = 'Be'
+  ename_table(5)  = 'B'
+  ename_table(6)  = 'C'
+  ename_table(7)  = 'N'
+  ename_table(8)  = 'O'
+  ename_table(9)  = 'F'
+  ename_table(10) = 'Ne'
+  ename_table(11) = 'Na'
+  ename_table(12) = 'Mg'
+  ename_table(13) = 'Al'
+  ename_table(14) = 'Si'
+  ename_table(15) = 'P'
+  ename_table(16) = 'S'
+  ename_table(17) = 'Cl'
+  ename_table(18) = 'Ar'
+  ename_table(19) = 'K'
+  ename_table(20) = 'Ca'
+  ename_table(21) = 'Sc'
+  ename_table(22) = 'Ti'
+  ename_table(23) = 'V'
+  ename_table(24) = 'Cr'
+  ename_table(25) = 'Mn'
+  ename_table(26) = 'Fe'
+  ename_table(27) = 'Co'
+  ename_table(28) = 'Ni'
+  ename_table(29) = 'Cu'
+  ename_table(30) = 'Zn'
+  ename_table(31) = 'Ga'
+  ename_table(32) = 'Ge'
+  ename_table(33) = 'As'
+  ename_table(34) = 'Se'
+  ename_table(35) = 'Br'
+  ename_table(36) = 'Kr'
+  ename_table(37) = 'Rb'
+  ename_table(38) = 'Sr'
+  ename_table(39) = 'Y'
+  ename_table(40) = 'Zr'
+  ename_table(41) = 'Nb'
+  ename_table(42) = 'Mo'
+  ename_table(43) = 'Tc'
+  ename_table(44) = 'Ru'
+  ename_table(45) = 'Rh'
+  ename_table(46) = 'Pd'
+  ename_table(47) = 'Ag'
+  ename_table(48) = 'Cd'
+  ename_table(49) = 'In'
+  ename_table(50) = 'Sn'
+  ename_table(51) = 'Sb'
+  ename_table(52) = 'Te'
+  ename_table(53) = 'I'
+  ename_table(54) = 'Xe'
+  ename_table(55) = 'Cs'
+  ename_table(56) = 'Ba'
+  ename_table(57) = 'La'
+  ename_table(58) = 'Ce'
+  ename_table(59) = 'Pr'
+  ename_table(60) = 'Nd'
+  ename_table(61) = 'Pm'
+  ename_table(62) = 'Sm'
+  ename_table(63) = 'Eu'
+  ename_table(64) = 'Gd'
+  ename_table(65) = 'Tb'
+  ename_table(66) = 'Dy'
+  ename_table(67) = 'Ho'
+  ename_table(68) = 'Er'
+  ename_table(69) = 'Tm'
+  ename_table(70) = 'Yb'
+  ename_table(71) = 'Lu'
+  ename_table(72) = 'Hf'
+  ename_table(73) = 'Ta'
+  ename_table(74) = 'W'
+  ename_table(75) = 'Re'
+  ename_table(76) = 'Os'
+  ename_table(77) = 'Ir'
+  ename_table(78) = 'Pt'
+  ename_table(79) = 'Au'
+  ename_table(80) = 'Hg'
+  ename_table(81) = 'Tl'
+  ename_table(82) = 'Pb'
+  ename_table(83) = 'Bi'
+  ename_table(84) = 'Po'
+  ename_table(85) = 'At'
+  ename_table(86) = 'Rn'
+  ename_table(87) = 'Fr'
+  ename_table(88) = 'Ra'
+  ename_table(89) = 'Ac'
+  ename_table(90) = 'Th'
+  ename_table(91) = 'Pa'
+  ename_table(92) = 'U'
+  ename_table(93) = 'Np'
+  ename_table(94) = 'Pu'
+  ename_table(95) = 'Am'
+  ename_table(96) = 'Cm'
+  ename_table(97) = 'Bk'
+  ename_table(98) = 'Cf'
+  ename_table(99) = 'Es'
+  ename_table(100) = 'Fm'
+  ename_table(101) = 'Md'
+  ename_table(102) = 'No'
+  ename_table(103) = 'Lr'
+  ename_table(104) = 'Rf'
+  ename_table(105) = 'Db'
+  ename_table(106) = 'Sg'
+  ename_table(107) = 'Bh'
+  ename_table(108) = 'Hs'
+  ename_table(109) = 'Mt'
+  ename_table(110) = 'Ds'
+  ename_table(111) = 'Rg'
+  ename_table(112) = 'Cn'
+  ename_table(113) = 'Nh'
+  ename_table(114) = 'Fl'
+  ename_table(115) = 'Mc'
+  ename_table(116) = 'Lv'
+  ename_table(117) = 'Ts'
+  ename_table(118) = 'Og'
+
+  iatomn = 0 
+  do it=1,ntyp
+     do ie=1,nmax_elem 
+        if(trim(speciesname(it)) == trim(ename_table(ie))) then
+           iatomn(it) = real(ie)
+        else
+           pos = index(trim(speciesname(it)),trim(ename_table(ie)))
+           if (pos.eq.1) then
+              read(speciesname(it)(len(trim(ename_table(ie)))+1:len(speciesname(it))),*,err=90,end=90) ii
+              iatomn(it) = real(ie)
+90            continue
+           endif
+        endif
+     enddo
+  enddo
+  deallocate(ename_table) 
+
+end subroutine set_iatomn_default
+
+subroutine set_covrad_default(ntyp, iatomn, covrads)
+  use m_Const_Parameters, only : DP,BOHR
+  implicit none
+!
+  integer, intent(in) :: ntyp
+  real(kind=DP), intent(in) :: iatomn(ntyp)
+  real(kind=DP), intent(inout) :: covrads(ntyp)
+  real(kind=DP) :: ang2bohr
+  real(kind=DP), allocatable, dimension(:) :: cov_table
+!
+  integer :: it,i
+  integer :: nmax_elem = 120
+  ang2bohr = 1.d0/BOHR
+  allocate(cov_table(nmax_elem))
+
+  cov_table(:) = 1.d0
+
+  cov_table(1) = 0.32d0
+  cov_table(2) = 0.93d0
+  cov_table(3) = 1.23d0
+  cov_table(4) = 0.9d0
+  cov_table(5) = 0.82d0
+  cov_table(6) = 0.77d0
+  cov_table(7) = 0.75d0
+  cov_table(8) = 0.73d0
+  cov_table(9) = 0.72d0
+  cov_table(10) = 0.71d0
+  cov_table(11) = 1.54d0
+  cov_table(12) = 1.36d0
+  cov_table(13) = 1.18d0
+  cov_table(14) = 1.11d0
+  cov_table(15) = 1.06d0
+  cov_table(16) = 1.02d0
+  cov_table(17) = 0.99d0
+  cov_table(18) = 0.98d0
+  cov_table(19) = 2.03d0
+  cov_table(20) = 1.74d0
+  cov_table(21) = 1.44d0
+  cov_table(22) = 1.32d0
+  cov_table(23) = 1.22d0
+  cov_table(24) = 1.18d0
+  cov_table(25) = 1.17d0
+  cov_table(26) = 1.17d0
+  cov_table(27) = 1.16d0
+  cov_table(28) = 1.15d0
+  cov_table(29) = 1.17d0
+  cov_table(30) = 1.25d0
+  cov_table(31) = 1.26d0
+  cov_table(32) = 1.22d0
+  cov_table(33) = 1.2d0
+  cov_table(34) = 1.16d0
+  cov_table(35) = 1.14d0
+  cov_table(36) = 1.89d0
+  cov_table(37) = 2.16d0
+  cov_table(38) = 1.91d0
+  cov_table(39) = 1.62d0
+  cov_table(40) = 1.45d0
+  cov_table(41) = 1.34d0
+  cov_table(42) = 1.3d0
+  cov_table(43) = 1.27d0
+  cov_table(44) = 1.25d0
+  cov_table(45) = 1.25d0
+  cov_table(46) = 1.28d0
+  cov_table(47) = 1.34d0
+  cov_table(48) = 1.41d0
+  cov_table(49) = 1.44d0
+  cov_table(50) = 1.41d0
+  cov_table(51) = 1.4d0
+  cov_table(52) = 1.36d0
+  cov_table(53) = 1.33d0
+  cov_table(54) = 1.31d0
+  cov_table(55) = 2.35d0
+  cov_table(56) = 1.98d0
+  cov_table(57) = 1.25d0
+  cov_table(58) = 1.65d0
+  cov_table(59) = 1.65d0
+  cov_table(60) = 1.64d0
+  cov_table(61) = 1.63d0
+  cov_table(62) = 1.62d0
+  cov_table(63) = 1.85d0
+  cov_table(64) = 1.61d0
+  cov_table(65) = 1.59d0
+  cov_table(66) = 1.59d0
+  cov_table(67) = 1.58d0
+  cov_table(68) = 1.57d0
+  cov_table(69) = 1.56d0
+  cov_table(70) = 1.7d0
+  cov_table(71) = 1.56d0
+  cov_table(72) = 1.44d0
+  cov_table(73) = 1.34d0
+  cov_table(74) = 1.37d0
+  cov_table(75) = 1.28d0
+  cov_table(76) = 1.26d0
+  cov_table(77) = 1.27d0
+  cov_table(78) = 1.3d0
+  cov_table(79) = 1.34d0
+  cov_table(80) = 1.49d0
+  cov_table(81) = 1.48d0
+  cov_table(82) = 1.47d0
+  cov_table(83) = 1.46d0
+  cov_table(84) = 1.53d0
+  cov_table(85) = 1.47d0
+  cov_table(86) = 1.42d0
+  cov_table(87) = 2.23d0
+  cov_table(88) = 2.01d0
+  cov_table(89) = 1.86d0
+  cov_table(90) = 1.75d0
+  cov_table(91) = 1.69d0
+  cov_table(92) = 1.7d0
+  cov_table(93) = 1.71d0
+  cov_table(94) = 1.72d0
+  cov_table(95) = 1.66d0
+  cov_table(96) = 1.66d0
+  cov_table(97) = 1.66d0
+  cov_table(98) = 1.68d0
+  cov_table(99) = 1.65d0
+  cov_table(100) = 1.67d0
+  cov_table(101) = 1.73d0
+  cov_table(102) = 1.76d0
+  cov_table(103) = 1.61d0
+  cov_table(104) = 1.57d0
+  cov_table(105) = 1.49d0
+  cov_table(106) = 1.43d0
+  cov_table(107) = 1.41d0
+  cov_table(108) = 1.34d0
+  cov_table(109) = 1.29d0
+
+  do it=1,ntyp
+     i = nint( iatomn(it) )
+     covrads(it) = cov_table(i) * ang2bohr
+  enddo
+
+  deallocate(cov_table)
+
+end subroutine set_covrad_default
+
+! === KT_add ======= 13.3B
+!*** ref. http://www.caslab.com/List-of-Elements/ *****
+!
+subroutine set_amion_default( ntyp, iatomn, amion )
+  use m_Const_Parameters, only : DP
+  implicit none
+!
+  integer, intent(in) :: ntyp
+  real(kind=DP), intent(in) :: iatomn(ntyp)
+  real(kind=DP), intent(inout) :: amion(ntyp)
+!
+  integer :: nmax_elem = 120
+!
+  integer :: it, i
+  real(kind=DP) :: coeff
+  real(kind=DP), allocatable :: mass_table(:)
+!
+  allocate( mass_table( nmax_elem ) )
+
+  call set_table(   1, 1.00794d0, 4.002602d0, 6.941d0, 9.012182d0, 10.811d0 )
+  call set_table(   6, 12.0107d0, 14.0067d0, 15.9994d0, 18.9984032d0, 20.1797d0 )
+  call set_table(  11, 22.98976928d0, 24.3050d0, 26.9815386d0, 28.0855d0, 30.973762d0 )
+  call set_table(  16, 32.065d0, 35.453d0, 39.948d0, 39.0983d0, 40.078d0 )
+
+  call set_table(  21, 44.955912d0, 47.867d0, 50.9415d0, 51.9961d0, 54.938045d0 )
+  call set_table(  26, 55.845d0, 58.933195d0, 58.6934d0, 63.546d0, 65.409d0 )
+  call set_table(  31, 69.723d0, 72.64d0, 74.92160d0, 78.96d0, 79.904d0 )
+  call set_table(  36, 83.798d0, 85.4678d0, 87.62d0, 88.90585d0, 91.224d0 )
+
+  call set_table(  41, 92.90638d0, 95.94d0, 97.9072d0, 101.07d0, 102.90550d0 )
+  call set_table(  46, 106.42d0, 107.8682d0, 112.411d0, 114.818d0, 118.710d0 )
+  call set_table(  51, 121.760d0, 127.60d0, 126.90447d0, 131.293d0, 132.9054519d0 )
+  call set_table(  56, 137.327d0, 138.90547d0, 140.116d0, 140.90765d0, 144.242d0 )
+
+  call set_table(  61, 144.9127d0, 150.36d0, 151.964d0, 157.25d0, 158.92535d0 )
+  call set_table(  66, 162.500d0, 164.93032d0, 167.259d0, 168.93421d0, 173.04d0 )
+  call set_table(  71, 174.967d0, 178.49d0, 180.94788d0, 183.84d0, 186.207d0 )
+  call set_table(  76, 190.23d0, 192.217d0, 195.084d0, 196.966569d0, 200.59d0 )
+
+  call set_table(  81, 204.3833d0, 207.2d0, 208.98040d0, 208.9824d0, 209.9871d0 )
+  call set_table(  86, 222.0176d0, 223.0197d0, 226.0254d0, 227.0277d0, 232.03806d0 )
+  call set_table(  91, 231.03588d0, 238.02891d0, 237.0482d0, 244.0642d0, 243.0614d0 )
+  call set_table(  96, 247.0704d0, 247.0703d0, 251.0796d0, 252.0830d0, 257.0951d0 )
+
+  call set_table( 101, 258.0984d0, 259.1010d0, 262.1097d0, 261.1088d0, 262d0 )
+  call set_table( 106, 266d0, 264d0, 277d0, 268d0, 271d0 )
+  call set_table( 111, 272d0, 285d0, 284d0, 289d0, 288d0 )
+  call set_table( 116, 292d0, 293d0, 294d0, 500d0, 500d0 )
+
+!
+  coeff = 1.66053d-27 /9.1093897d-31
+  Do it=1, ntyp
+     if ( amion(it) < -99.0 ) then
+        i = nint( iatomn(it) )
+        amion(it) = mass_table(i) *coeff
+     endif
+  End do
+
+  deallocate( mass_table )
+
+contains
+
+  subroutine set_table( istart, c1, c2, c3, c4, c5 )
+    integer, intent(in) :: istart
+    real(kind=DP), intent(in) :: c1, c2, c3, c4, c5
+
+    mass_table( istart ) = c1
+    mass_table( istart +1 ) = c2
+    mass_table( istart +2 ) = c3
+    mass_table( istart +3 ) = c4
+    mass_table( istart +4 ) = c5
+  end subroutine set_table
+
+end subroutine set_amion_default
+! ========== 13.3B
+
+subroutine get_ac_from_dynm(nfdynm,targetframe,natm,nchar,ntyp,avec,bvec,cvec,speciesname,ityp,cps,cpd)
+  implicit none
+  integer, intent(in) :: nfdynm,targetframe,natm,nchar,ntyp
+  real(kind=8), dimension(3), intent(out) :: avec,bvec,cvec
+  character(len=nchar), dimension(ntyp), intent(out) :: speciesname
+  integer, dimension(natm), intent(out) :: ityp
+  real(kind=8), dimension(natm,3), intent(out) :: cps,cpd
+  integer, parameter :: ncharline=256
+  integer, parameter :: ncharword=20
+  integer :: nwords = 20
+  character(len=ncharline) :: line
+  character(len=ncharword), allocatable, dimension(:) :: output
+  integer :: i
+  integer :: tmpint
+  logical :: in_header, in_data
+  integer :: ntypc,currat,iframe
+  integer :: nword_out
+
+  allocate(output(nwords))
+  in_header = .false.
+  in_data = .false.
+  ntypc = 0
+  iframe = 0
+  avec=0.d0;bvec=0.d0;cvec=0.d0
+  speciesname='';ityp=0.d0;cps=0.d0
+  do 
+    read(unit=nfdynm,end=100,fmt='(a)') line
+    call split(ncharline,ncharword,nwords,line,output,nword_out)
+    if(.not. in_header .and. trim(output(1)) == '#')  then
+      in_header = .true.
+      cycle
+    endif
+    if(in_header) then
+      if(trim(output(2)) == 'a_vector')then
+        ntypc = 0
+        currat = 0
+        read(output(4),*) avec(1)
+        read(output(5),*) avec(2)
+        read(output(6),*) avec(3)
+      else if(trim(output(2)) == 'b_vector')then
+        read(output(4),*) bvec(1)
+        read(output(5),*) bvec(2)
+        read(output(6),*) bvec(3)
+      else if(trim(output(2)) == 'c_vector')then
+        read(output(4),*) cvec(1)
+        read(output(5),*) cvec(2)
+        read(output(6),*) cvec(3)
+      else if(trim(output(2)) == '(natm->type)')then
+        do i=3,nwords
+          if(len(trim(output(i)))==0) exit
+          ntypc = ntypc + 1
+          read(output(i),*) ityp(ntypc)
+        enddo
+      else if(trim(output(2)) == '(speciesname)')then
+        read(output(3),*) tmpint 
+        speciesname(tmpint)(1:nchar) = output(5)(1:nchar)
+      endif
+    endif
+    if(output(1) == 'cps')then
+      in_header = .false.
+      in_data = .true.
+      ntypc = 0
+      currat = 0
+      cycle
+    endif
+    if(in_data)then
+      currat = currat+1
+      read(output(2),*) cps(currat,1)
+      read(output(3),*) cps(currat,2)
+      read(output(4),*) cps(currat,3)
+      if(nword_out == 10)then
+        read(output(5),*) cpd(currat,1)
+        read(output(6),*) cpd(currat,2)
+        read(output(7),*) cpd(currat,3)
+      endif
+      if(currat == natm)then
+        iframe = iframe+1
+        in_data = .false.
+        if(targetframe .eq.iframe) then
+           exit
+        endif
+      endif
+    endif
+  enddo
+
+100 continue
+  deallocate(output)
+end subroutine get_ac_from_dynm
+
+subroutine get_numat_and_ntyp(nfdynm,natm,ntyp)
+  implicit none
+  integer, intent(in) :: nfdynm
+  integer, intent(out) :: natm,ntyp
+  integer, parameter :: ncharline=256
+  integer, parameter :: ncharword=20
+  integer :: nwords = 20
+  integer :: nword_out
+  character(len=ncharline) :: line
+  character(len=ncharword), allocatable, dimension(:) :: output
+  logical :: in_header
+  integer :: i
+
+  natm = 0;ntyp = 0
+  allocate(output(nwords))
+  in_header = .false.
+  do 
+    read(unit=nfdynm,end=100,fmt='(a)') line
+    call split(ncharline,ncharword,nwords,line,output,nword_out)
+    if(.not. in_header .and. trim(output(1)) == '#')  then
+      in_header = .true.
+      cycle
+    endif
+    if(in_header) then
+      if(trim(output(2)) == 'ntyp')then
+        read(output(4),*) ntyp
+        read(output(7),*) natm
+        return
+      endif
+    endif
+  enddo
+100 continue
+  write(0,*) '!** failed to resolve natm and ntyp'
+  return
+end subroutine get_numat_and_ntyp
+
+subroutine split(ncharline,ncharword,nwords,input,output,nword_out) 
+  implicit none
+  integer, intent(in) :: ncharline,ncharword,nwords
+  character(len=ncharline), intent(in) :: input
+  character(len=ncharword), dimension(nwords), intent(out) :: output
+  integer, intent(out) :: nword_out
+  integer :: i, currword,currind
+  logical :: was_ws
+  was_ws = .true.
+  currword = 0
+  currind = 0
+  do i=1,nwords
+    output(i) = ''
+  enddo
+  do i=1,ncharline
+    if (input(i:i) .eq. ' ') then
+      was_ws = .true.
+      cycle
+    endif
+    if (was_ws) then
+      currword = currword + 1
+      was_ws = .false.
+      currind = 0
+    endif
+    currind = currind+1
+    output(currword)(currind:currind) = input(i:i)
+  enddo
+  nword_out = currword
+end subroutine split
+
